/* /////////////////////////////////////////////////////////////////////////////
 * File:        unittest.cpp
 *
 * Purpose:		Unit test file for WinSTL
 *
 * Created:		11th April 2004
 * Updated:     27th August 2005
 *
 * Author:      Matthew Wilson
 *
 * Copyright:   Synesis Software Pty Ltd, (c) 2002. All rights reserved.
 *
 * Home:        www.stlsoft.org
 *
 * ////////////////////////////////////////////////////////////////////////// */


// Must define this
#ifndef STLSOFT_UNITTEST
# error This file cannot be used if STLSOFT_UNITTEST is not defined.
#endif /* !STLSOFT_UNITTEST */

#if defined(__MWERKS__)
# if defined(_STLSOFT_COMPILE_VERBOSE)
#  undef _STLSOFT_COMPILE_VERBOSE
# endif /* _STLSOFT_COMPILE_VERBOSE */
#endif /* __MWERKS__ */

#ifdef __BORLANDC__
# pragma warn -8008
# pragma warn -8066
#endif /* __BORLANDC__ */

#include <stdio.h>
#include <stlsoft.h>
#include <winstl.h>
#if !defined(__DMC__)
# include <ntsecapi.h>
#endif /* compiler */
#include <unittest/simple_unittester.h>

#if defined(__STLSOFT_COMPILER_IS_INTEL)
# pragma warning(disable : 1419)
# if !defined(__MWERKS__)
#  define __MWERKS__	(0)
# endif /* !__MWERKS__ */
# if !defined(__DMC__)
#  define __DMC__	(0)
# endif /* !__DMC__ */
#endif /* !__STLSOFT_COMPILER_IS_MSVC */

////////////////////////////////////////////////////////////////////////////////
// main

//extern "C" void __stdcall Sleep(unsigned long);

int main(int argc, char **argv)
{
//	Sleep(100000);

	int		i;
	bool	bVerbose	=	false;

	for(i = 1; i < argc; ++i)
	{
		if(	0 == strcmp(argv[i], "-v") ||
			0 == strcmp(argv[i], "-verbose"))
		{
			bVerbose = true;
		}
	}

	puts("Running WinSTL unittests");

	return !::stlsoft::unittest::simple_unittest_host::get_host()->test(bVerbose);
}

////////////////////////////////////////////////////////////////////////////////

#if defined(__INTEL_COMPILER)
# if !defined(_WIN32_WINNT)
#  define _WIN32_WINNT	0
# endif /* !_WIN32_WINNT */
#endif /* __INTEL_COMPILER */

#if !defined(_WIN32_WINNT) || \
    (_WIN32_WINNT < 0x0500)

static FARPROC GetKernel32ProcOrTerminate(LPSTR procName)
{
	// We're incrementing KERNEL32's reference count, but that's ok,
	// because it's KERNEL32. Not to be recommended normally.
	FARPROC	fp	=	::GetProcAddress(::LoadLibrary("Kernel32"), procName);

	if(NULL == fp)
	{
		::ExitProcess(1);
	}

	return fp;
}

HANDLE WINAPI FindFirstVolumeA(LPSTR lpszVolumeName, DWORD cchBufferLength);
HANDLE WINAPI FindFirstVolumeA(LPSTR lpszVolumeName, DWORD cchBufferLength)
{
	return (*reinterpret_cast<HANDLE (WINAPI *)(LPSTR , DWORD )>(GetKernel32ProcOrTerminate("FindFirstVolumeA")))(lpszVolumeName, cchBufferLength);
}

HANDLE WINAPI FindFirstVolumeW(LPWSTR lpszVolumeName, DWORD cchBufferLength);
HANDLE WINAPI FindFirstVolumeW(LPWSTR lpszVolumeName, DWORD cchBufferLength)
{
	return (*reinterpret_cast<HANDLE (WINAPI *)(LPWSTR , DWORD )>(GetKernel32ProcOrTerminate("FindFirstVolumeW")))(lpszVolumeName, cchBufferLength);
}

BOOL WINAPI FindNextVolumeA(HANDLE hFindVolume, LPSTR lpszVolumeName, DWORD cchBufferLength);
BOOL WINAPI FindNextVolumeA(HANDLE hFindVolume, LPSTR lpszVolumeName, DWORD cchBufferLength)
{
	return (*reinterpret_cast<BOOL (WINAPI *)(HANDLE, LPSTR , DWORD )>(GetKernel32ProcOrTerminate("FindNextVolumeA")))(hFindVolume, lpszVolumeName, cchBufferLength);
}

BOOL WINAPI FindNextVolumeW(HANDLE hFindVolume, LPWSTR lpszVolumeName, DWORD cchBufferLength);
BOOL WINAPI FindNextVolumeW(HANDLE hFindVolume, LPWSTR lpszVolumeName, DWORD cchBufferLength)
{
	return (*reinterpret_cast<BOOL (WINAPI *)(HANDLE, LPWSTR , DWORD )>(GetKernel32ProcOrTerminate("FindNextVolumeW")))(hFindVolume, lpszVolumeName, cchBufferLength);
}

BOOL WINAPI FindVolumeClose(HANDLE hFindVolume);
BOOL WINAPI FindVolumeClose(HANDLE hFindVolume)
{
	return (*reinterpret_cast<BOOL (WINAPI *)(HANDLE)>(GetKernel32ProcOrTerminate("FindVolumeClose")))(hFindVolume);
}

#endif /* !_WIN32_WINNT || (_WIN32_WINNT < 0x0500) */

////////////////////////////////////////////////////////////////////////////////

#ifdef STLSOFT_COMPILER_IS_DMC

static FARPROC GetPsapiProcOrTerminate(LPSTR procName)
{
	// We're incrementing PSAPI's reference count, but we'll live with it.
	// Not to be recommended normally.
	FARPROC	fp	=	::GetProcAddress(::LoadLibrary("PSAPI"), procName);

	if(NULL == fp)
	{
		fprintf(stderr, "Cannot dynamically load PSAPI\n");
		fflush(stderr);

		::ExitProcess(1);
	}

	return fp;
}

BOOL
WINAPI
EnumProcesses(
  DWORD *lpidProcess,  /* array of process identifiers */
  DWORD cb,            /* size of array */
  DWORD *cbNeeded      /* number of bytes returned */
);
BOOL
WINAPI
EnumProcesses(
  DWORD *lpidProcess,  /* array of process identifiers */
  DWORD cb,            /* size of array */
  DWORD *cbNeeded      /* number of bytes returned */
)
{
	return (*reinterpret_cast<BOOL (WINAPI *)(DWORD*, DWORD, DWORD*)>(GetPsapiProcOrTerminate("EnumProcesses")))(lpidProcess, cb, cbNeeded);
}

BOOL
WINAPI
EnumProcessModules(
  HANDLE hProcess,      /* handle to process */
  HMODULE *lphModule,   /* array of module handles */
  DWORD cb,             /* size of array */
  LPDWORD lpcbNeeded    /* number of bytes required */
);
BOOL
WINAPI
EnumProcessModules(
  HANDLE hProcess,      /* handle to process */
  HMODULE *lphModule,   /* array of module handles */
  DWORD cb,             /* size of array */
  LPDWORD lpcbNeeded    /* number of bytes required */
)
{
	return (*reinterpret_cast<BOOL (WINAPI *)(HANDLE, HMODULE*, DWORD, DWORD*)>(GetPsapiProcOrTerminate("EnumProcessModules")))(hProcess, lphModule, cb, lpcbNeeded);
}

#endif /* STLSOFT_COMPILER_IS_DMC */

////////////////////////////////////////////////////////////////////////////////
